import * as mc from "@minecraft/server"
import {
     ActionFormData,
     MessageFormData,
     ModalFormData
}
from "@minecraft/server-ui"
import {
     CommandPrefix,
     OpenSetting,
     OpenEntitySetting,
     OpenBlockSetting,
     OpenPlayerSetting,
     OpenDisplaySetting,
     Reload,
     UILatency,
     DisplayHelp
} from "../config"

import {
     isPhone_1,
     isPhone_2,
     isPc_1,
     isPc_2
} from "../look/displayDevice"

export function setting() {
     mc.world.beforeEvents.chatSend.subscribe(system => {
          let text = system.message
          let player = system.sender
          // 打开生物设置（与玩家不是一起的）
          if (new RegExp(CommandPrefix + OpenEntitySetting, "i").test(text)) {
               system.cancel = true
               mc.system.runTimeout(() => {
                    entitySettingUi(player)
               }, UILatency)
          }

          // 打开方块设置
          if (new RegExp(CommandPrefix + OpenBlockSetting, "i").test(text)) {
               system.cancel = true
               mc.system.runTimeout(() => {
                    blockSettingUi(player)
               }, UILatency)
          }

          // 打开玩家设置
          if (new RegExp(CommandPrefix + OpenPlayerSetting, "i").test(text)) {
               system.cancel = true
               mc.system.runTimeout(() => {
                    playerSettingUi(player)
               }, UILatency)
          }

          // 打开设置
          if (new RegExp(CommandPrefix + OpenSetting, "i").test(text)) {
               system.cancel = true
               mc.system.runTimeout(() => {
                    settingUi(player)
               }, UILatency)
          }

          // 打开显示设置
          if (new RegExp(CommandPrefix + OpenDisplaySetting, "i").test(text)) {
               system.cancel = true
               mc.system.runTimeout(() => {
                    displayUi(player)
               }, UILatency)
          }

          // 打开帮助
          if (new RegExp(CommandPrefix + DisplayHelp, "i").test(text)) {
               system.cancel = true
               mc.system.runTimeout(() => {
                    help(player)
               })
          }

          //重置
          if (new RegExp(CommandPrefix + Reload, "i").test(text)) {
               system.cancel = true
               mc.system.run(() => {
                    player.clearDynamicProperties()
                    player.sendMessage({
                         text: 'All settings have been reset'
                    })
               })
          }
     })
}
export function initialize(player) {
     const initialProperties = {
          '返回': false,
          '生物距离': 10,
          '方块距离': 10,
          '残血': 30
     }

     for (const property in initialProperties) {
          if (player.getDynamicProperty(property) === undefined) {
               player.setDynamicProperty(property, initialProperties[property])
          }
     }
}

function settingUi(player) {
     player.setDynamicProperty('返回', false)
     let playerIcon
     if (player.name == 'Xuxinrui5201314') {
          playerIcon = 'icon/奶糖'
     } else if (player.name == 'MaxCypher') {
          playerIcon = 'icon/MaxCypher'
     } else {
          playerIcon = 'icon/player'
     }
     const a = new ActionFormData()
          .title('Settings')
          .button('Player Settings', playerIcon)
          .button('Entity Settings', 'icon/entity')
          .button('Block Settings', 'icon/block')
          .button('Display Settings', 'icon/display')
          .button('Reset All Settings', 'icon/forget_book')
          .show(player)
          .then(response => {
               if (response.canceled) {
                    return
               } else {
                    if (response.selection == 0) {
                         playerSettingUi(player)
                         player.setDynamicProperty('返回', true)
                    }
                    if (response.selection == 1) {
                         entitySettingUi(player)
                         player.setDynamicProperty('返回', true)
                    }
                    if (response.selection == 2) {
                         blockSettingUi(player)
                         player.setDynamicProperty('返回', true)
                    }

                    if (response.selection == 3) {
                         displayUi(player)
                         player.setDynamicProperty('返回', true)
                    }

                    if (response.selection == 4) {
                         reset(player)
                    }
               }
          })
}

function displayUi(player) {
     const a = new ModalFormData()
          .title('Display Settings')
          .slider('Entity Display Distance', 5, 80, 5, player.getDynamicProperty('生物距离'))
          .slider('Block Display Distance', 5, 80, 5, player.getDynamicProperty('方块距离'))
          .slider('Low Health Display (Percentage)', 0, 90, 10, player.getDynamicProperty('残血'))
          .toggle(`Show Player Info (Recommended)`, player.getDynamicProperty('玩家信息'))
          .toggle(`Show Entity Info (Recommended)`, player.getDynamicProperty('生物信息'))
          .toggle(`Show Block Info (Recommended)`, player.getDynamicProperty('方块信息'))
          .show(player)
          .then(response => {
               if (response.canceled) {
                    if (player.getDynamicProperty('返回')) {
                         settingUi(player)
                         player.setDynamicProperty('返回', false)
                    } else {
                         return
                    }
               } else {
                    const displaySetting = [
                         '生物距离',
                         '方块距离',
                         '残血',
                         '玩家信息',
                         '生物信息',
                         '方块信息'
                    ]
                    response.formValues.forEach((value, index) => {
                         player.setDynamicProperty(displaySetting[index], value)
                    })
               }
          })
}

function reset(player) {

     const a = new MessageFormData()
          .body(`This action is irreversible, are you sure you want to reset all settings?`)
          .button1('Cancel')
          .button2('Confirm')
          .show(player)
          .then(response => {
               if (response.selection == 0) {
                    settingUi(player)
               }
               if (response.selection == 1) {
                    player.clearDynamicProperties()
                    player.sendMessage({
                         text: 'All settings have been reset'
                    })
               }
          })
}
export function entityInitialize(player) {
     const initialProperties = {
          '生物生命值': true,
          '使用爱心': true,
          '是幼崽': true,
          '驯服': true,
          '生物模组归属': true,
          '目标': true,
          '生物信息': true,
          '移动速度': true,
          '水下移动速度': true,
          '能否牵引': true,
          '能否驯服': true,
          '生物标识符': true
     }

     for (const property in initialProperties) {
          if (player.getDynamicProperty(property) === undefined) {
               player.setDynamicProperty(property, initialProperties[property])
          }
     }
}

export function blockInitialize(player) {
     const initialProperties = {
          '能否采集': true,
          '采集等级': true,
          '采集工具': true,
          '红石状态': true,
          '红石能量': true,
          '中继器': true,
          '比较器': true,
          '漏斗': true,
          '告示牌': true,
          '台阶': true,
          '容器': true,
          '植物': true,
          '植物百分比': false,
          '可疑沙子': true,
          '可疑沙子百分比': false,
          '朝向': true,
          '脚手架': true,
          '蜂蜜': true,
          '蜂蜜百分比': false,
          '重生锚': true,
          '堆肥箱': true,
          '堆肥箱百分比': false,
          '炼药锅容量': true,
          '炼药锅容量百分比': false,
          '炼药锅装有': true,
          '固体': true,
          '浸水': true,
          '三原色': true,
          '方块标识符': true,
          '方块坐标': true,
          '方块模组归属': true,
          '方块信息': true
     }

     for (const property in initialProperties) {
          if (player.getDynamicProperty(property) === undefined) {
               player.setDynamicProperty(property, initialProperties[property])
          }
     }
}

export function playerInitialize(player) {
     const a = player.name == 'MaxCypher' ? 1 : 0
     const b = player.name == 'MaxCypher' ? 1 : 0
     const initialProperties = {
          '设备信息': true,
          '手机图案': a,
          '电脑图案': b,
          '管理员状态': true,
          '玩家模式': true,
          '玩家血量': true,
          '玩家护甲值': true,
          '不死图腾': true,
          '盾牌': true,
          '等级': true,
          '药水': true,
          '玩家信息': true
     }
     for (const property in initialProperties) {
          if (player.getDynamicProperty(property) === undefined) {
               player.setDynamicProperty(property, initialProperties[property])
          }
     }
}

function entitySettingUi(player) {
     const a = new ModalFormData()
          .title('Entity Settings')
          .toggle(`Show Health (Recommended)`, player.getDynamicProperty('生物生命值'))
          .toggle(`Show Heart Health (Only effective for entities with 20 health)`, player.getDynamicProperty('使用爱心'))
          .toggle(`Show Baby Entities`, player.getDynamicProperty('是幼崽'))
          .toggle(`Show Tamed Entities`, player.getDynamicProperty('驯服'))
          .toggle(`Show Addon Source`, player.getDynamicProperty('生物模组归属'))
          .toggle(`Show Target of Entity's Attack`, player.getDynamicProperty('目标'))
          .toggle(`Show Movement Speed`, player.getDynamicProperty('移动速度'))
          .toggle(`Show Underwater Movement Speed`, player.getDynamicProperty('水下移动速度'))
          .toggle(`Show Whether Can Be Pulled`, player.getDynamicProperty('能否牵引'))
          .toggle(`Show Whether Can Be Tamed`, player.getDynamicProperty('能否驯服'))
          .toggle(`Show Identifier`, player.getDynamicProperty('生物标识符'))
          .toggle(`Show Entity Information (Recommended)`, player.getDynamicProperty('生物信息'))
          .show(player)
          .then(response => {
               if (response.canceled) {
                    if (player.getDynamicProperty('返回')) {
                         settingUi(player)
                         player.setDynamicProperty('返回', false)
                    } else {
                         return
                    }
               } else {
                    const entitySetting = [
                         '生物生命值',
                         '使用爱心',
                         '是幼崽',
                         '驯服',
                         '生物模组归属',
                         '目标',
                         '移动速度',
                         '水下移动速度',
                         '能否牵引',
                         '能否驯服',
                         '生物标识符',
                         '生物信息'
                    ]
                    response.formValues.forEach((value, index) => {
                         player.setDynamicProperty(entitySetting[index], value)
                    })
               }
          })
}

function blockSettingUi(player) {
     const a = new ModalFormData()
          .title('Block Settings')
          .toggle(`Show if Mined`, player.getDynamicProperty('能否采集'))
          .toggle(`Show Best Enchantment Level`, player.getDynamicProperty('采集等级'))
          .toggle(`Show Mining Tool`, player.getDynamicProperty('采集工具'))
          .toggle(`Show Redstone State (e.g. Levers, Trapdoors)`, player.getDynamicProperty('红石状态'))
          .toggle(`Show Redstone Power`, player.getDynamicProperty('红石能量'))
          .toggle(`Show Repeater Delay`, player.getDynamicProperty('中继器'))
          .toggle(`Show Comparator State`, player.getDynamicProperty('比较器'))
          .toggle(`Show Hopper State (Only Shown When Disabled)`, player.getDynamicProperty('漏斗'))
          .toggle(`Show Sign Text`, player.getDynamicProperty('告示牌'))
          .toggle(`Show Slab (Only Shown When Top Half)`, player.getDynamicProperty('台阶'))
          .toggle(`Show Container Capacity`, player.getDynamicProperty('容器'))
          .toggle(`Show Plant Stage`, player.getDynamicProperty('植物'))
          .toggle(`Show Plant Growth Percentage`, player.getDynamicProperty('植物百分比'))
          .toggle(`Show Suspicious Sand Progress`, player.getDynamicProperty('可疑沙子'))
          .toggle(`Show Suspicious Sand Progress Percentage`, player.getDynamicProperty('可疑沙子百分比'))
          .toggle(`Show Block Facing`, player.getDynamicProperty('朝向'))
          .toggle(`Show Scaffolding Durability`, player.getDynamicProperty('脚手架'))
          .toggle(`Show Honey Capacity`, player.getDynamicProperty('蜂蜜'))
          .toggle(`Show Honey Capacity Percentage`, player.getDynamicProperty('蜂蜜百分比'))
          .toggle(`Show Resurrection Anchor Charge`, player.getDynamicProperty('重生锚'))
          .toggle(`Show Composter Fill Level`, player.getDynamicProperty('堆肥箱'))
          .toggle(`Show Composter Fill Percentage`, player.getDynamicProperty('堆肥箱百分比'))
          .toggle(`Show Brewing Stand Capacity`, player.getDynamicProperty('炼药锅容量'))
          .toggle(`Show Brewing Stand Capacity Percentage`, player.getDynamicProperty('炼药锅容量百分比'))
          .toggle(`Show What's in the Brewing Stand (Default: Water)`, player.getDynamicProperty('炼药锅装有'))
          .toggle(`Show if Solid`, player.getDynamicProperty('固体'))
          .toggle(`Show if Wet`, player.getDynamicProperty('浸水'))
          .toggle(`Show Primary Colors`, player.getDynamicProperty('三原色'))
          .toggle(`Show Identifier`, player.getDynamicProperty('方块标识符'))
          .toggle(`Show Block Coordinates`, player.getDynamicProperty('方块坐标'))
          .toggle(`Show Addon Source`, player.getDynamicProperty('方块模组归属'))
          .toggle(`Show Block Information (Recommended)`, player.getDynamicProperty('方块信息'))
          .show(player)
          .then(response => {
               if (response.canceled) {
                    if (player.getDynamicProperty('返回')) {
                         settingUi(player)
                         player.setDynamicProperty('返回', false)
                    } else {
                         return
                    }
               } else {
                    const blockSetting = [
                         '能否采集',
                         '采集等级',
                         '采集工具',
                         '红石状态',
                         '红石能量',
                         '中继器',
                         '比较器',
                         '漏斗',
                         '告示牌',
                         '台阶',
                         '容器',
                         '植物',
                         '植物百分比',
                         '可疑沙子',
                         '可疑沙子百分比',
                         '朝向',
                         '脚手架',
                         '蜂蜜',
                         '蜂蜜百分比',
                         '重生锚',
                         '堆肥箱',
                         '堆肥箱百分比',
                         '炼药锅容量',
                         '炼药锅容量百分比',
                         '炼药锅装有',
                         '固体',
                         '浸水',
                         '三原色',
                         '方块标识符',
                         '方块坐标',
                         '方块模组归属',
                         '方块信息'
                    ]
                    response.formValues.forEach((value, index) => {
                         player.setDynamicProperty(blockSetting[index], value)
                    })
               }
          })
}

function playerSettingUi(player) {
     const phoneIcon = [isPhone_1, isPhone_2]
     const pcIcon = [isPc_1, isPc_2]
     const a = new ModalFormData()
          .title('Player Settings')
          .toggle(`Show Device Information`, player.getDynamicProperty('设备信息'))
          .dropdown("Phone Pattern Style", phoneIcon, player.getDynamicProperty('手机图案'))
          .dropdown("PC Pattern Style", pcIcon, player.getDynamicProperty('电脑图案'))
          .toggle(`Show Admin Status`, player.getDynamicProperty('管理员状态'))
          .toggle(`Show Player Mode`, player.getDynamicProperty('玩家模式'))
          .toggle(`Show Health`, player.getDynamicProperty('玩家血量'))
          .toggle(`Show Armor`, player.getDynamicProperty('玩家护甲值'))
          .toggle(`Show Totem of Undying`, player.getDynamicProperty('不死图腾'))
          .toggle(`Show Shield`, player.getDynamicProperty('盾牌'))
          .toggle(`Show Level`, player.getDynamicProperty('等级'))
          .toggle(`Show Potion Effects`, player.getDynamicProperty('药水'))
          .toggle(`Player Information (Recommended)`, player.getDynamicProperty('玩家信息'))
          .show(player)
          .then(response => {
               if (response.canceled) {
                    if (player.getDynamicProperty('返回')) {
                         settingUi(player)
                         player.setDynamicProperty('返回', false)
                    } else {
                         return
                    }
               } else {
                    const playerSetting = [
                         '设备信息',
                         '手机图案',
                         '电脑图案',
                         '管理员状态',
                         '玩家模式',
                         '玩家血量',
                         '玩家护甲值',
                         '不死图腾',
                         '盾牌',
                         '等级',
                         '药水',
                         '玩家信息'
                    ]
                    response.formValues.forEach((value, index) => {
                         player.setDynamicProperty(playerSetting[index], value)
                    })
               }
          })
}

export function help(player) {
     player.sendMessage({
          rawtext: [{
                    text: 'Help for the "What Can I See" Addon\n'
               },
               {
                    text: '——————————————————————————————————\n'
               },
               {
                    text: `Type §b${CommandPrefix}${OpenSetting}§r to open the settings menu\n`
               },
               {
                    text: `Type §b${CommandPrefix}${OpenPlayerSetting}§r to open the player settings\n`
               },
               {
                    text: `Type §b${CommandPrefix}${OpenEntitySetting}§r to open the mob settings\n`
               },
               {
                    text: `Type §b${CommandPrefix}${OpenBlockSetting}§r to open the block settings\n`
               },
               {
                    text: `Type §b${CommandPrefix}${OpenDisplaySetting}§r to open the display settings\n`
               },
               {
                    text: `Type §b${CommandPrefix}${DisplayHelp}§r to get some help\n`
               },
               {
                    text: `Type §b${CommandPrefix}${Reload}§r to reset your settings\n`
               },
               {
                    text: `§l§cCase insensitive§r\n`
               },
               {
                    text: '——————————————————————————————————'
               }
          ]
     })
}